@@ -9,14 +9,15 @@
  */
 
 #ifndef lint
-static char id[] = "@(#)$Id: handler.c,v 8.19 2000/02/11 00:12:29 ca Exp $";
+static char id[] = "@(#)$Id: handler.c,v 8.19.4.2 2000/07/14 06:16:57 msk Exp $";
 #endif /* ! lint */
 
 #if _FFR_MILTER
 #include "libmilter.h"
 
+
 /*
-**  HANDLE_SESSION -- Handle a connected session in it's own context
+**  HANDLE_SESSION -- Handle a connected session in its own context
 **
 **	Parameters:
 **		ctx -- context structure
@@ -33,7 +34,7 @@ mi_handle_session(ctx)
 
 	if (ctx == NULL)
 		return MI_FAILURE;
-	ctx->ctx_id = pthread_self();
+	ctx->ctx_id = (sthread_t) sthread_get_id();
 
 	/*
 	**  detach so resources are free when the thread returns
@@ -42,8 +43,8 @@ mi_handle_session(ctx)
 	if (pthread_detach(ctx->ctx_id) != 0)
 		return MI_FAILURE;
 	ret = mi_engine(ctx);
-	if (ctx->ctx_fd >= 0)
-		(void) close(ctx->ctx_fd);
+	if (ValidSocket(ctx->ctx_sd))
+		(void) close(ctx->ctx_sd);
 	if (ctx->ctx_reply != NULL)
 		free(ctx->ctx_reply);
 	if (ctx->ctx_privdata != NULL)
