@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998, 1999 Sendmail, Inc. and its suppliers.
+ * Copyright (c) 1998-2000 Sendmail, Inc. and its suppliers.
  *	All rights reserved.
  * Copyright (c) 1983 Eric P. Allman.  All rights reserved.
  * Copyright (c) 1988, 1993
@@ -21,7 +21,7 @@ static char copyright[] =
 #endif /* ! lint */
 
 #ifndef lint
-static char id[] = "@(#)$Id: mailstats.c,v 8.53 1999/10/13 05:43:54 gshapiro Exp $";
+static char id[] = "@(#)$Id: mailstats.c,v 8.53.16.10 2000/07/18 05:51:15 gshapiro Exp $";
 #endif /* ! lint */
 
 #include <unistd.h>
@@ -39,8 +39,10 @@ static char id[] = "@(#)$Id: mailstats.c,v 8.53 1999/10/13 05:43:54 gshapiro Exp
 #include <sendmail/mailstats.h>
 #include <sendmail/pathnames.h>
 
+
 #define MNAMELEN	20	/* max length of mailer name */
 
+
 int
 main(argc, argv)
 	int argc;
@@ -66,11 +68,12 @@ main(argc, argv)
 	extern char *optarg;
 	extern int optind;
 
+
 	cfile = _PATH_SENDMAILCF;
 	sfile = NULL;
 	mnames = TRUE;
 	progmode = FALSE;
-	while ((ch = getopt(argc, argv, "C:f:op")) != EOF)
+	while ((ch = getopt(argc, argv, "C:f:op")) != -1)
 	{
 		switch (ch)
 		{
@@ -93,7 +96,7 @@ main(argc, argv)
 		  case '?':
 		  default:
   usage:
-			(void) fputs("usage: mailstats [-C cffile] [-f stfile] -o -p\n",
+			(void) fputs("usage: mailstats [-C cffile] [-f stfile] [-o] [-p]\n",
 				     stderr);
 			exit(EX_USAGE);
 		}
@@ -202,8 +205,8 @@ main(argc, argv)
 		exit (EX_OSFILE);
 	}
 
-	if ((fd = open(sfile, O_RDONLY)) < 0 ||
-	    (i = read(fd, &stats, sizeof stats)) < 0)
+	fd = open(sfile, O_RDONLY);
+	if ((fd < 0) || (i = read(fd, &stats, sizeof stats)) < 0)
 	{
 		save_errno = errno;
 		(void) fputs("mailstats: ", stderr);
