@@ -12,7 +12,7 @@
  */
 
 #ifndef lint
-static char id[] = "@(#)$Id: headers.c,v 8.203 2000/03/15 21:47:29 ca Exp $";
+static char id[] = "@(#)$Id: headers.c,v 8.203.4.6 2000/07/19 02:53:32 ca Exp $";
 #endif /* ! lint */
 
 #include <sendmail.h>
@@ -70,7 +70,7 @@ static struct hdrinfo	NormalHeader =	{ NULL, 0, NULL };
 u_long
 chompheader(line, pflag, hdrp, e)
 	char *line;
-	int *pflag;
+	int pflag;
 	HDR **hdrp;
 	register ENVELOPE *e;
 {
@@ -81,6 +81,7 @@ chompheader(line, pflag, hdrp, e)
 	char *fname;
 	char *fvalue;
 	bool cond = FALSE;
+	bool dropfrom;
 	bool headeronly;
 	STAB *s;
 	struct hdrinfo *hi;
@@ -101,7 +102,7 @@ chompheader(line, pflag, hdrp, e)
 	/* strip off options */
 	clrbitmap(mopts);
 	p = line;
-	if (!bitset(*pflag, CHHDR_USER) && *p == '?')
+	if (!bitset(pflag, CHHDR_USER) && *p == '?')
 	{
 		int c;
 		register char *q;
@@ -204,7 +205,7 @@ hse:
 		return H_EOH;
 
 	/* check to see if it represents a ruleset call */
-	if (bitset(*pflag, CHHDR_DEF))
+	if (bitset(pflag, CHHDR_DEF))
 	{
 		char hbuf[50];
 
@@ -249,12 +250,12 @@ hse:
 	}
 
 	/* see if this is a resent message */
-	if (!bitset(*pflag, CHHDR_DEF) && !headeronly &&
+	if (!bitset(pflag, CHHDR_DEF) && !headeronly &&
 	    bitset(H_RESENT, hi->hi_flags))
 		e->e_flags |= EF_RESENT;
 
 	/* if this is an Errors-To: header keep track of it now */
-	if (UseErrorsTo && !bitset(*pflag, CHHDR_DEF) && !headeronly &&
+	if (UseErrorsTo && !bitset(pflag, CHHDR_DEF) && !headeronly &&
 	    bitset(H_ERRORSTO, hi->hi_flags))
 		(void) sendtolist(fvalue, NULLADDR, &e->e_errorqueue, 0, e);
 
@@ -280,7 +281,7 @@ hse:
 	**  If there is a check ruleset, verify it against the header.
 	*/
 
-	if (bitset(*pflag, CHHDR_CHECK))
+	if (bitset(pflag, CHHDR_CHECK))
 	{
 		bool stripcom = FALSE;
 		char *rs;
@@ -349,52 +350,17 @@ hse:
 		}
 	}
 
-#if _FFR_MILTER
-	/* Call milter */
-	if (bitset(*pflag, CHHDR_MILTER) &&
-	    !bitset(EF_DISCARD, e->e_flags))
-	{
-		char state;
-		char *response;
-
-		response = milter_header(fname, fvalue, e, &state);
-		switch (state)
-		{
-		  case SMFIR_REPLYCODE:
-			*pflag &= ~CHHDR_MILTER;
-			usrerr(response);
-			break;
-
-		  case SMFIR_REJECT:
-			*pflag &= ~CHHDR_MILTER;
-			usrerr("554 5.7.1 Message rejected");
-			break;
-
-		  case SMFIR_DISCARD:
-			*pflag &= ~CHHDR_MILTER;
-			e->e_flags |= EF_DISCARD;
-			break;
-
-		  case SMFIR_TEMPFAIL:
-			*pflag &= ~CHHDR_MILTER;
-			usrerr("451 4.7.1 Try again later");
-			break;
-		}
-		if (response != NULL)
-			free(response);
-	}
-#endif /* _FFR_MILTER */
-
 	/*
 	**  Drop explicit From: if same as what we would generate.
 	**  This is to make MH (which doesn't always give a full name)
 	**  insert the full name information in all circumstances.
 	*/
 
+	dropfrom = FALSE;
 	p = "resent-from";
 	if (!bitset(EF_RESENT, e->e_flags))
 		p += 7;
-	if (!bitset(*pflag, CHHDR_DEF) && !headeronly &&
+	if (!bitset(pflag, CHHDR_DEF) && !headeronly &&
 	    !bitset(EF_QUEUERUN, e->e_flags) && strcasecmp(fname, p) == 0)
 	{
 		if (tTd(31, 2))
@@ -407,14 +373,14 @@ hse:
 		    bitnset(M_LOCALMAILER, e->e_from.q_mailer->m_flags) &&
 		    (strcmp(fvalue, e->e_from.q_paddr) == 0 ||
 		     strcmp(fvalue, e->e_from.q_user) == 0))
-			return hi->hi_flags;
+			dropfrom = TRUE;
 	}
 
 	/* delete default value for this header */
 	for (hp = hdrp; (h = *hp) != NULL; hp = &h->h_link)
 	{
 		if (strcasecmp(fname, h->h_field) == 0 &&
-		    bitset(H_DEFAULT, h->h_flags) &&
+		    !bitset(H_USER, h->h_flags) &&
 		    !bitset(H_FORCE, h->h_flags))
 		{
 			if (nullheader)
@@ -422,6 +388,12 @@ hse:
 				/* user-supplied value was null */
 				return 0;
 			}
+			if (dropfrom)
+			{
+				/* make this look like the user entered it */
+				h->h_flags |= H_USER;
+				return hi->hi_flags;
+			}
 			h->h_value = NULL;
 			if (!cond)
 			{
@@ -442,17 +414,19 @@ hse:
 	h->h_macro = mid;
 	*hp = h;
 	h->h_flags = hi->hi_flags;
+	if (bitset(pflag, CHHDR_USER))
+		h->h_flags |= H_USER;
 
 	/* strip EOH flag if parsing MIME headers */
 	if (headeronly)
 		h->h_flags &= ~H_EOH;
-	if (bitset(*pflag, CHHDR_DEF))
+	if (bitset(pflag, CHHDR_DEF))
 		h->h_flags |= H_DEFAULT;
 	if (cond || mid != '\0')
 		h->h_flags |= H_CHECK;
 
 	/* hack to see if this is a new format message */
-	if (!bitset(*pflag, CHHDR_DEF) && !headeronly &&
+	if (!bitset(pflag, CHHDR_DEF) && !headeronly &&
 	    bitset(H_RCPT|H_FROM, h->h_flags) &&
 	    (strchr(fvalue, ',') != NULL || strchr(fvalue, '(') != NULL ||
 	     strchr(fvalue, '<') != NULL || strchr(fvalue, ';') != NULL))
@@ -470,6 +444,7 @@ hse:
 **	Parameters:
 **		field -- the name of the header field.
 **		value -- the value of the field.
+**		flags -- flags to add to h_flags.
 **		hdrlist -- an indirect pointer to the header structure list.
 **
 **	Returns:
@@ -480,9 +455,10 @@ hse:
 */
 
 void
-addheader(field, value, hdrlist)
+addheader(field, value, flags, hdrlist)
 	char *field;
 	char *value;
+	int flags;
 	HDR **hdrlist;
 {
 	register HDR *h;
@@ -504,7 +480,7 @@ addheader(field, value, hdrlist)
 	h->h_field = field;
 	h->h_value = newstr(value);
 	h->h_link = *hp;
-	h->h_flags = H_DEFAULT;
+	h->h_flags = flags;
 	if (s != NULL)
 		h->h_flags |= s->s_header.hi_flags;
 	clrbitmap(h->h_mflags);
@@ -917,7 +893,7 @@ logsender(e, msgid)
 	p = macvalue(macid("{auth_type}", NULL), e);
 	if (p != NULL)
 	{
-		(void) snprintf(sbp, SPACELEFT(sbuf, sbp), ", mech=%.10s", p);
+		(void) snprintf(sbp, SPACELEFT(sbuf, sbp), ", mech=%.12s", p);
 		sbp += strlen(sbp);
 	}
 	p = macvalue(macid("{auth_author}", NULL), e);
@@ -982,7 +958,7 @@ priencode(p)
 
 	for (i = 0; i < NumPriorities; i++)
 	{
-		if (!strcasecmp(p, Priorities[i].pri_name))
+		if (strcasecmp(p, Priorities[i].pri_name) == 0)
 			return Priorities[i].pri_val;
 	}
 
@@ -1405,6 +1381,10 @@ putheader(mci, hdr, e, flags)
 			xputs(p);
 		}
 
+		/* Skip empty headers */
+		if (h->h_value == NULL)
+			continue;
+
 		/* heuristic shortening of MIME fields to avoid MUA overflows */
 		if (MaxMimeFieldLength > 0 &&
 		    wordinclass(h->h_field,
@@ -1425,7 +1405,7 @@ putheader(mci, hdr, e, flags)
 		    wordinclass(h->h_field,
 				macid("{checkMIMETextHeaders}", NULL)))
 		{
-			if (strlen(h->h_value) > MaxMimeHeaderLength)
+			if (strlen(h->h_value) > (size_t)MaxMimeHeaderLength)
 			{
 				h->h_value[MaxMimeHeaderLength - 1] = '\0';
 				sm_syslog(LOG_ALERT, e->e_id,
@@ -1612,7 +1592,7 @@ put_vanilla_header(h, v, mci)
 		int l;
 
 		l = nlp - v;
-		if (SPACELEFT(obuf, obp) - 1 < l)
+		if (SPACELEFT(obuf, obp) - 1 < (size_t)l)
 			l = SPACELEFT(obuf, obp) - 1;
 
 		snprintf(obp, SPACELEFT(obuf, obp), "%.*s", l, v);
